function rscontour(y, D, lims, prefix)
% RSCONTOUR - Draw a contour plot for a quadratic two-factor response surface.
%             The output is saved in pdf format on request. Version 0.1.23.
%
% The syntax is
% rscontour(y, D, [lims], [prefix])
%
% Arguments
% y is an N by 1 column vector of measured response data.
% D is an N by 2 design matrix with data for two factors.
% [lims] is a voluntary 4 by 1 vector of plot limits [x1min x1max x2min x2max]. Defaults are the maximum and minimum values in D.
% [prefix] is a voluntary parameter, a string which causes rscontour to save the plot as a pdf file. [prefix] should be a string which will be prepended to the file name of the pdf file.
% 
% If you want to save a PDF with the default plot limits, put an empty matrix [] for lims.
%
% References
% Myers, Montgomery & Anderson-Cook: Response Surface Methodology, 3rd ed., Wiley, 2009, Section 6.3.4.


% Copyright (C) 2016 Mikael Kurula <mkurula@abo.fi>
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 3 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
% FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, see <http://www.gnu.org/licenses/>.
%
% Author: Mikael Kurula <mkurula@abo.fi>, 2016


% Check input
if nargin < 2 || nargin > 4,
    error('Try help rscontour for instructions.') ;
end
if size(y,2) > 1,
    error('y has more than one column. Try help rscontour for instructions.');
end
if size(D,2) ~= 2,
    error('D must have two columns. Try help rscontour for instructions.');
end
if nargin > 2 && any(size(lims) ~= [1 4]) && any(size(lims) ~= [0 0]),
    error('lims must be a 1 by 4 vector or empty matrix. Try help rscontour for instructions.');
end

% Take care of differences between Octave and Matlab
if exist ('OCTAVE_VERSION', 'builtin')       % Are we running Octave?
   graphics_toolkit('gnuplot')
   PprSz=[6 4];
else
   PprSz=[12 8];
end 

% Do linear regression
N=size(D,1);
X=rsquad(D);
if cond(X'*X) > 1e10,
    warning('The normal equation is ill-conditioned. If the matrix D does not have linearly dependent columns or a column of ones, try standardizing the factor data.');
end
b=X\y;

% Determine plot limits and build prediction
steps=50;

if nargin == 2 || size(lims,2)==0,
  mn1=min(D(:,1));
  mx1=max(D(:,1));
  mn2=min(D(:,2));
  mx2=max(D(:,2));
else
  mn1=lims(1);
  mx1=lims(2);
  mn2=lims(3);
  mx2=lims(4);
end
stp1=(mx1-mn1)/steps;
stp2=(mx2-mn2)/steps;

for n=1:(steps+1)
  for m=1:(steps+1)
    xx1=(mx1-mn1)*(n-1)/steps+mn1;
    xx2=(mx2-mn2)*(m-1)/steps+mn2;
    yhat(m,n)=rsquad([xx1 xx2])*b;
  end
end

mny=min(min(yhat));
mxy=max(max(yhat));
deltay=(mxy-mny)/10;

% Plot contour lines
figure('PaperPosition',[0 0 PprSz],'PaperSize',PprSz);
c=contour(mn1:stp1:mx1,mn2:stp2:mx2,yhat,mny:deltay:mxy,'linewidth',2);
clabel (c, mny:deltay:mxy);
title('Contour plot of prediction by fitted quadratic model')
xlabel('x1')
ylabel('x2')
if nargin == 4 && size(prefix,1)>0,
   print([prefix,'-rscontour'],'-dpdf')
end

end